@@ -8,7 +8,7 @@ import pickle
 
 import numpy as np
 import optuna
-from sklearn.metrics import roc_auc_score
+from sklearn.metrics import roc_auc_score, mean_squared_error
 from sklearn.model_selection import cross_val_score
 
 import hipe4ml.tree_handler
@@ -17,12 +17,12 @@ import hipe4ml.tree_handler
 class ModelHandler:
     """
     Class used for wrapping the models from different ML libraries to
-    build a new model with common methods. Currently only XGBoost
-    (through it's sklearn interface) and sklearn models are supported.
+    build a new model with common methods. Currently LightGBM, XGBoost
+    (through their sklearn interface) and sklearn models are supported.
 
     Parameters
     -------------------------------------------------
-    input_model: xgboost or sklearn model
+    input_model: XGBoost, LightGBM or sklearn model
 
     training_columns: list
         Contains the name of the features used for the training.
@@ -32,16 +32,29 @@ class ModelHandler:
         Model hyper-parameter values. For
         example (XGBoost): max_depth, learning_rate,
         n_estimators, gamma, min_child_weight, ...
+
+    task_type: str
+        Task type of the model: 'classification' or 'regression'
     """
 
-    def __init__(self, input_model=None, training_columns=None, model_params=None):
+    def __init__(self, input_model=None, training_columns=None, model_params=None, task_type='classification'):
         self.model = input_model
         self.training_columns = training_columns
         self.model_params = model_params
         self._n_classes = None
+        self.optuna_cross_val_score = None
+        self._task_type = task_type
+        if self._task_type not in ['classification', 'regression']:
+            raise ValueError(
+                "Task type must be either 'classification' or 'regression'")
 
         if self.model is not None:
-            self.model_string = inspect.getmodule(self.model).__name__.partition('.')[0]
+            self.model_string = inspect.getmodule(
+                self.model).__name__.partition('.')[0]
+
+            if self.model_string not in ["xgboost", "lightgbm", "sklearn"]:
+                raise ValueError(
+                    "Model must be either 'xgboost', 'lightgbm' or 'sklearn'")
 
             if self.model_params is None:
                 self.model_params = self.model.get_params()
@@ -106,7 +119,7 @@ class ModelHandler:
 
         Returns
         ---------------------------
-        out: sklearn or XGBoost model
+        out: XGBoost, LGBM or sklearn model
         """
         return self.model
 
@@ -133,6 +146,17 @@ class ModelHandler:
         """
         return self._n_classes
 
+    def get_task_type(self):
+        """
+        Get the task type of the model
+
+        Returns
+        ---------------------------
+        out: str
+            Task type of the model: 'classification' or 'regression'
+        """
+        return self._task_type
+
     def fit(self, x_train, y_train, **kwargs):
         """
         Fit Model
@@ -148,8 +172,9 @@ class ModelHandler:
         **kwargs:
             Extra kwargs passed on to model.fit() method
         """
-        n_classes = len(np.unique(y_train))
-        self._n_classes = n_classes
+        if self._task_type == 'classification':
+            n_classes = len(np.unique(y_train))
+            self._n_classes = n_classes
         if self.training_columns is None:
             self.training_columns = list(x_train.columns)
 
@@ -158,7 +183,6 @@ class ModelHandler:
     def predict(self, x_test, output_margin=True, **kwargs):
         """
         Return model prediction for the array x_test
-
         Parameters
         --------------------------------------
         x_test: hipe4ml tree_handler, array-like, sparse matrix
@@ -166,13 +190,15 @@ class ModelHandler:
 
         output_margin: bool
             Whether to output the raw untransformed margin value. If False model
-            probabilities are returned
+            probabilities are returned. Not used when task_type is 'regression'.
 
         **kwargs:
-            Extra kwargs passed on to the model prediction function:
-            - predict() (xgboost) or decision_function() (sklearn) if output_margin==True
+            Extra kwargs passed on to the following model prediction function:
+            if (task_type == 'classification')
+            - predict() (XGBoost and LGBM) or decision_function() (sklearn) if output_margin==True
             - predict_proba() if output_margin==False
-
+            if (task_type == 'regression')
+            - predict()
 
         Returns
         ---------------------------------------
@@ -184,25 +210,34 @@ class ModelHandler:
 
         x_test = x_test[self.training_columns]
 
+        # regression
+        if self._task_type == 'regression':
+            return self.model.predict(x_test, **kwargs)
+
+        # classification
         if output_margin:
             if self.model_string == 'xgboost':
-                pred = self.model.predict(x_test, True, **kwargs)
+                return self.model.predict(x_test, output_margin=True, **kwargs)
+            if self.model_string == 'lightgbm':
+                return self.model.predict(x_test, raw_score=True, **kwargs)
             if self.model_string == 'sklearn':
-                pred = self.model.decision_function(x_test, **kwargs).ravel()
-        else:
-            pred = self.model.predict_proba(x_test, **kwargs)
-            # in case of binary classification return only the scores of
-            # the signal class
-            if pred.shape[1] <= 2:
-                pred = pred[:, 1]
+                if not hasattr(self.model, 'decision_function'):
+                    raise ValueError(
+                        "This Model does not support a decision_function(): use output_margin=False")
+                return self.model.decision_function(x_test, **kwargs).ravel()
 
+        pred = self.model.predict_proba(x_test, **kwargs)
+        # in case of binary classification return only the scores of
+        # the signal class
+        if pred.shape[1] <= 2:
+            pred = pred[:, 1]
         return pred
 
     def train_test_model(self, data, return_prediction=False, output_margin=False, average='macro',
                          multi_class_opt='raise', **kwargs):
         """
         Perform the training and the testing of the model. The model performance is estimated
-        using the ROC AUC metric
+        using the ROC AUC metric for classification and the MSE for regression.
 
         Parameters
         ----------------------------------------------
@@ -217,7 +252,7 @@ class ModelHandler:
 
         output_margin: bool
             Whether to output the raw untransformed margin value. If False model
-            probabilities are returned
+            probabilities are returned. Not used when task_type is 'regression'.
 
         average: string
             Option for the average of ROC AUC scores used only in case of multi-classification.
@@ -242,19 +277,25 @@ class ModelHandler:
         # get number of classes
         n_classes = len(np.unique(data[1]))
         self._n_classes = n_classes
-        print('Number of detected classes:', n_classes)
+        print('==============================')
+        print(f"Training {self.model_string} model for {self._task_type}")
+        if self._task_type == 'classification':
+            print('Number of detected classes:', n_classes)
 
         # final training with the optimized hyperparams
-        print('Training the final model: ...', end='\r')
+        print('Training the model: ...')
         self.fit(data[0], data[1], **kwargs)
-        print('Training the final model: Done!')
-        print('Testing the model: ...', end='\r')
+        print('Training the model: Done!')
+        print('Testing the model: ...')
         y_pred = self.predict(data[2], output_margin=output_margin)
-        roc_score = roc_auc_score(
-            data[3], y_pred, average=average, multi_class=multi_class_opt)
+        if self._task_type == 'classification':
+            roc_score = roc_auc_score(
+                data[3], y_pred, average=average, multi_class=multi_class_opt)
+            print(f'ROC_AUC_score: {roc_score:.6f}')
+        else:
+            mse_score = mean_squared_error(data[3], y_pred)
+            print(f'Mean squared error: {mse_score:.6f}')
         print('Testing the model: Done!')
-
-        print(f'ROC_AUC_score: {roc_score:.6f}')
         print('==============================')
         if return_prediction:
             return y_pred
@@ -342,9 +383,11 @@ class ModelHandler:
 
             for key in hyperparams_ranges:
                 if isinstance(hyperparams_ranges[key][0], int):
-                    params[key] = trial.suggest_int(key, hyperparams_ranges[key][0], hyperparams_ranges[key][1])
+                    params[key] = trial.suggest_int(
+                        key, hyperparams_ranges[key][0], hyperparams_ranges[key][1])
                 elif isinstance(hyperparams_ranges[key][0], float):
-                    params[key] = trial.suggest_uniform(key, hyperparams_ranges[key][0], hyperparams_ranges[key][1])
+                    params[key] = trial.suggest_uniform(
+                        key, hyperparams_ranges[key][0], hyperparams_ranges[key][1])
 
             return params
 
@@ -359,7 +402,8 @@ class ModelHandler:
             with open(resume_study, 'rb') as resume_study_file:
                 study = pickle.load(resume_study_file)
         else:
-            study = optuna.create_study(direction=direction, sampler=optuna_sampler)
+            study = optuna.create_study(
+                direction=direction, sampler=optuna_sampler)
 
         study.optimize(__objective, **kwargs)
 
@@ -377,6 +421,7 @@ class ModelHandler:
             print(f"    {key}: {value}")
 
         self.set_model_params({**self.model_params, **best_trial.params})
+        self.optuna_cross_val_score = best_trial.value
 
         return study
 
@@ -432,3 +477,4 @@ class ModelHandler:
             self.model.set_params(**self.model_params)
             self.model_string = loaded_model.get_model_module()
             self._n_classes = loaded_model.get_n_classes()
+            self._task_type = loaded_model.get_task_type()
