@@ -15,7 +15,7 @@ class TreeHandler:
     or a pandas.DataFrame from a .parquet file
     """
 
-    def __init__(self, file_name=None, tree_name=None, columns_names=None, **kwds):
+    def __init__(self, file_name=None, tree_name=None, column_names=None, folder_name=None, **kwds):
         """
         Open the file in which the selected tree leaves are converted
         into pandas dataframe columns. If tree_name is not provided file_name is
@@ -30,31 +30,74 @@ class TreeHandler:
             Name of the tree within the input file, must be the same for all files.
             If None the method pandas.read_parquet is called
 
-        columns_name: list
-            List of the names of the branches that one wants to analyse. If columns_names is
+        column_names: list
+            List of the names of the branches that one wants to analyse. If column_names is
             not specified all the branches are converted
 
+        folder_name: str
+            Name of the folder/folders within the input file. If the folder_name ends with a '*' all the folders
+            containing the string folder_name are read and merged into a single dataframe.
+            Example: folder_name = "DF*" will read all the folders containing the string "DF" and
+            merge them into a single dataframe.
+
         **kwds: extra arguments are passed on to the uproot.TTree.arrays() or pandas.read_parquet() methods:
                 https://uproot.readthedocs.io/en/latest/uproot.behaviors.TTree.TTree.html#uproot.behaviors.TTree.TTree.arrays
                 https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.read_parquet.html#pandas.read_parquet
         """
         self._tree = tree_name
         self._full_data_frame = None
-        if file_name is not None:
-            self._full_data_frame = pd.DataFrame()
-            self._files = file_name if isinstance(file_name, list) else [file_name]
-            for file in self._files:
-                if self._tree is not None:
-                    self._full_data_frame = self._full_data_frame.append(
-                        uproot.open(f'{file}:{self._tree}').arrays(filter_name=columns_names, library='pd', **kwds),
-                        ignore_index=True)
-                else:
-                    self._full_data_frame = self._full_data_frame.append(
-                        pd.read_parquet(file, columns=columns_names, **kwds), ignore_index=True)
         self._preselections = None
         self._projection_variable = None
         self._projection_binning = None
         self._sliced_df_list = None
+        if file_name is None:
+            return
+
+        self._full_data_frame = pd.DataFrame()
+        self._files = file_name if isinstance(file_name, list) else [file_name]
+        for file in self._files:
+            if self._tree is None:  # read from a parquet file
+                self._full_data_frame = pd.concat(
+                [self._full_data_frame, pd.read_parquet(file, columns=column_names, **kwds)],
+                ignore_index=True, copy=False)
+                continue
+
+            if folder_name is None:
+                self._full_data_frame = pd.concat(
+                    [self._full_data_frame,
+                        uproot.open(f'{file}:{self._tree}').arrays(filter_name=column_names,
+                        library='pd', **kwds)], ignore_index=True, copy=False)
+                continue
+
+            if folder_name[-1] != '*':
+                self._full_data_frame = pd.concat(
+                    [self._full_data_frame,
+                        uproot.open(f'{file}:{folder_name}/{self._tree}').arrays(filter_name=column_names,
+                        library='pd', **kwds)], ignore_index=True, copy=False)
+                continue
+
+            file_folders = uproot.open(file).keys()
+            # check if there are multiple cycles of the same tree, keep only last one
+            # first we sort to have as first one the last cycle
+            file_folders.sort(reverse=True)
+            file_folders_to_remove = []
+            for ifolder, folder in enumerate(file_folders[1:]):
+                obj_nocycle = folder.split(";")[0]
+                if obj_nocycle in file_folders[ifolder]:
+                    file_folders_to_remove.append(folder)
+            for folder_to_remove in file_folders_to_remove:
+                file_folders.remove(folder_to_remove)
+            tree_path_list = []
+            for folder in file_folders:
+                if folder_name[:-1] in folder and self._tree in folder:
+                    tree_path_list.append(folder)
+
+            for tree_path in tree_path_list:
+                print(f"Reading {file}:{tree_path}")
+                self._full_data_frame = pd.concat(
+                    [self._full_data_frame,
+                        uproot.open(f'{file}:{tree_path}').arrays(filter_name=column_names,
+                        library='pd', **kwds)], ignore_index=True, copy=False)
 
     def __getitem__(self, column):
         """
